//----------------------------------------------------------------------------

//Project:  B77 ROM Emulator
//Compiler: IAR EWAVR 6.12
//Target:   ATtiny2313
//Version:  1.0
//E-mail:   wubblick@yahoo.com
//Fuses:    EXT = 0xFF, HI = 0xDF, LO = 0xE4

//----------------------------------------------------------------------------

#include "Main.h"

//------------------------------- Options: -----------------------------------

//#define PAUSE_LOCK

//------------------------ Functions prototypes: -----------------------------

void main(void);                   //main program
void Main_Ports_Init(void);        //ports init
void Main_Switch(void);            //switch outputs
void Main_Int_Init(void);          //pin change interrupt init
#pragma vector = PCIN1_vect
__interrupt void Pin_Change(void); //pin change interrupt

//----------------------------------------------------------------------------
//---------------------------- Main program: ---------------------------------
//----------------------------------------------------------------------------

void main(void)
{
  Main_Ports_Init();         //ports init
  Main_Int_Init();           //pin change interrupt init
  Main_Switch();             //switch outputs
  __enable_interrupt();      //interrupts enable

  while(1)                   //main loop
  {
    __sleep();               //go to sleep
  }
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

//----------------------------- Ports init: ----------------------------------

void Main_Ports_Init(void)
{
  DDRA  = I_DDRA;
  PORTA = I_PORTA;
  DDRB  = I_DDRB;
  PORTB = I_PORTB;
  DDRD  = I_DDRD;
  PORTD = I_PORTD;
  ACSR = (1 << ACD);
}

//--------------------------- Switch outputs: --------------------------------

#pragma inline = forced
void Main_Switch(void)
{
  static bool Pause = 0;
#ifdef PAUSE_LOCK
  static bool KeyPause = 0;
  bool PinPause = Pin_PAUSE;    //read PAUSE pin
  if(KeyPause != PinPause)
  {
    KeyPause = PinPause;
    if(KeyPause) Pause = !Pause; //toggle PAUSE mode
  }
#else
  Pause = Pin_PAUSE;            //read PAUSE pin
#endif
  char Mode = PINB & MODE_MASK; //select mode bits
  switch(Mode)
  {
//------ STOP ------
  case MD_STOP:
    Pause = 0;
    Port_nRECRES_0; //REC FF RESET
    Port_nMVSET_1;  //MOVE FF RELEASE
    Port_RECON_0;   //REC OFF
    Port_PRESS_0;   //PRESS OFF
    Port_BRAKE_0;   //BRAKE
    Port_nPLAY_1;   //PLAY OFF
    Port_nREW_1;    //REW OFF
    Port_nFFD_1;    //FFD OFF
    break;

//------ REW ------
  case MD_REW:
    Port_nRECRES_0; //REC FF RESET
    Port_nMVSET_0;  //MOVE FF SET
    Port_RECON_0;   //REC OFF
    Port_PRESS_0;   //PRESS OFF
    Port_nPLAY_1;   //PLAY OFF
    Port_nFFD_1;    //FFD OFF
    if(Pause)       //if PAUSE
    {
      Port_BRAKE_0; //BRAKE
      Port_nREW_1;  //REW OFF
    }
    else
    {
      Port_BRAKE_1; //UNBRAKE
      Port_nREW_0;  //REW ON
    }
    break;

//------ FFD ------
  case MD_FFD:
    Port_nRECRES_0; //REC FF RESET
    Port_nMVSET_0;  //MOVE FF SET
    Port_RECON_0;   //REC OFF
    Port_PRESS_0;   //PRESS OFF
    Port_nPLAY_1;   //PLAY OFF
    Port_nREW_1;    //REW OFF
    if(Pause)       //if PAUSE
    {
      Port_BRAKE_0; //BRAKE
      Port_nFFD_1;  //FFD OFF
    }
    else            //else (NOT PAUSE)
    {
      Port_BRAKE_1; //UNBRAKE
      Port_nFFD_0;  //FFD ON
    }
    break;

//------ PLAY/REC ------
  case MD_PLAY:
    Port_nRECRES_1; //REC FF RELEASE
    Port_nMVSET_1;  //MOVE FF RELEASE
    Port_nREW_1;    //REW OFF
    Port_nFFD_1;    //FFD OFF
    if(Pause || Pin_MOVE) //if PAUSE or TAPE MOVES
    {
      Port_PRESS_0; //PRESS OFF
      Port_BRAKE_0; //BRAKE
      Port_nPLAY_1; //PLAY OFF
    }
    else //else (NOT PAUSE and TAPE NOT MOVES)
    {
      Port_BRAKE_1; //UNBRAKE
      Port_PRESS_1; //PRESS ON
      Port_nPLAY_0; //PLAY ON (TAKE UP)
    }
    if(Pin_REC && !Pin_MOVE) //REC and TAPE NOT MOVES
      Port_RECON_1;          //REC ON
        else Port_RECON_0;   //else REC OFF
    break;
  }
}

//---------------------- Pin change interrupt init: --------------------------

void Main_Int_Init(void)
{
  PCMSK = (1<<PCINT4) | (1<<PCINT3) | (1<<PCINT2) | (1<<PCINT1) | (1<<PCINT0);
  EIFR = (1 << PCIF);
  GIMSK = (1 << PCIE);
  MCUCR = (1 << SM1) | (1 << SM0) | (1 << SE); //power down enable
}

//------------------------- Pin change interrupt: ----------------------------

#pragma vector = PCIN1_vect
__interrupt void Pin_Change(void)
{
  Main_Switch();
}

//----------------------------------------------------------------------------
