//----------------------------------------------------------------------------

//B77 ROM Emulator, header file

//----------------------------------------------------------------------------

//Original ROM - N82S123 (32 x 8).
//Original code:
//2D 2D 8C 0D 3D 3D 9C 1D 89 0D E7 6D 99 1D F7 7D
//2D 2D 8C 0D 2D 2D 9C 1D 89 0D 6D 6D 99 1D 6D 6D

//Inputs:
//A0 - PAUSE
//A1 - MODE0
//A2 - REC
//A3 - MODE1
//A4 - MOVE

//Outputs:
//D0 - nREW
//D1 - PRESS
//D2 - nFFD
//D3 - nPLAY
//D4 - RECON
//D5 - nMVSET
//D6 - nRECRES
//D7 - BRAKE

//MODE0 = 0, MODE1 = 0 - STOP
//MODE0 = 1, MODE1 = 0 - REW
//MODE0 = 0, MODE1 = 1 - FFD
//MODE0 = 1, MODE1 = 1 - PLAY (REC)

//----------------------------------------------------------------------------

#ifndef MainH
#define MainH

//----------------------------------------------------------------------------

#include <iotiny2313.h>
#include <intrinsics.h>
#include <stdbool.h>

//----------------------------------------------------------------------------
//-------------------------------- Ports: ------------------------------------
//----------------------------------------------------------------------------

//Legend:

//I - input
//O - outbut
//B - bidirectional

//L - low active level
//H - high active level
//X - undefined active level
//A - analog signal

//------------------------------- Port A: ------------------------------------

#define nFFD    (1 << PA0) //OL - FFD output
#define nPLAY   (1 << PA1) //OL - PLAY output
#define NC_PA2  (1 << PA2) //IL - not used (RESET)

//Direction:
#define I_DDRA  (nFFD | nPLAY)
//Pull-ups (in) or initial state (out):
#define I_PORTA (nFFD | nPLAY | NC_PA2)
//Port control macros:
#define Port_nFFD_0    (PORTA &= ~nFFD)
#define Port_nFFD_1    (PORTA |= nFFD)
#define Port_nPLAY_0   (PORTA &= ~nPLAY)
#define Port_nPLAY_1   (PORTA |= nPLAY)

//------------------------------- Port B: ------------------------------------

#define PAUSE   (1 << PB0) //IH - PAUSE input
#define MODE0   (1 << PB1) //IH - MODE0 input
#define REC     (1 << PB2) //IH - REC input
#define MOVE    (1 << PB3) //IH - MOVE input
#define MODE1   (1 << PB4) //IH - MODE1 input
#define NC_PB5  (1 << PB5) //IL - not used (MOSI)
#define NC_PB6  (1 << PB6) //IL - not used (MISO)
#define NC_PB7  (1 << PB7) //IL - not used (SCK)

//Direction:
#define I_DDRB  (0)
//Pull-ups (in) or initial state (out):
#define I_PORTB (PAUSE | MODE0 | REC | MOVE | MODE1 | NC_PB5 | NC_PB6 | NC_PB7)
//Port control macros:
#define Pin_PAUSE      (PINB & PAUSE)
#define Pin_MODE0      (PINB & MODE0)
#define Pin_REC        (PINB & REC)
#define Pin_MOVE       (PINB & MOVE)
#define Pin_MODE1      (PINB & MODE1)

//------------------------------- Port D: ------------------------------------

#define nREW    (1 << PD0) //OL - REW output
#define PRESS   (1 << PD1) //OH - K-PRESS output
#define RECON   (1 << PD2) //OH - RECORD ON output
#define NC_PD3  (1 << PD3) //IL - not used
#define nMVSET  (1 << PD4) //OL - MOVE FF SET output
#define nRECRES (1 << PD5) //OL - RECORD FF RESET output
#define BRAKE   (1 << PD6) //OH - K-BRAKE output

//Direction:
#define I_DDRD  (nREW | PRESS | RECON | nMVSET | nRECRES | BRAKE)
//Pull-ups (in) or initial state (out):
#define I_PORTD (nREW | NC_PD3 | nMVSET | nRECRES)
//Port control macros:
#define Port_nREW_0    (PORTD &= ~nREW)
#define Port_nREW_1    (PORTD |= nREW)
#define Port_PRESS_0   (PORTD &= ~PRESS)
#define Port_PRESS_1   (PORTD |= PRESS)
#define Port_RECON_0   (PORTD &= ~RECON)
#define Port_RECON_1   (PORTD |= RECON)
#define Port_nMVSET_0  (PORTD &= ~nMVSET)
#define Port_nMVSET_1  (PORTD |= nMVSET)
#define Port_nRECRES_0 (PORTD &= ~nRECRES)
#define Port_nRECRES_1 (PORTD |= nRECRES)
#define Port_BRAKE_0   (PORTD &= ~BRAKE)
#define Port_BRAKE_1   (PORTD |= BRAKE)

//------------------------------- Constants: ---------------------------------

#define MODE_MASK (MODE0 | MODE1)
#define MD_STOP (0)
#define MD_REW  (MODE0)
#define MD_FFD  (MODE1)
#define MD_PLAY (MODE0 | MODE1)

//----------------------------------------------------------------------------

#endif
